@@ -16,7 +16,6 @@ from pyathena.util import RetryConfig
 
 if TYPE_CHECKING:
     from pyathena.connection import Connection
-    from pyathena.pandas.result_set import AthenaPandasResultSet
 
 _logger = logging.getLogger(__name__)  # type: ignore
 
@@ -25,34 +24,35 @@ class AsyncCursor(BaseCursor):
     def __init__(
         self,
         connection: "Connection",
-        s3_staging_dir: str,
-        poll_interval: float,
-        encryption_option: str,
-        kms_key: str,
         converter: Converter,
         formatter: Formatter,
         retry_config: RetryConfig,
+        s3_staging_dir: Optional[str] = None,
         schema_name: Optional[str] = None,
         catalog_name: Optional[str] = None,
         work_group: Optional[str] = None,
+        poll_interval: float = 1,
+        encryption_option: Optional[str] = None,
+        kms_key: Optional[str] = None,
+        kill_on_interrupt: bool = True,
         max_workers: int = (cpu_count() or 1) * 5,
         arraysize: int = CursorIterator.DEFAULT_FETCH_SIZE,
-        kill_on_interrupt: bool = True,
     ) -> None:
         super(AsyncCursor, self).__init__(
             connection=connection,
-            s3_staging_dir=s3_staging_dir,
-            poll_interval=poll_interval,
-            encryption_option=encryption_option,
-            kms_key=kms_key,
             converter=converter,
             formatter=formatter,
             retry_config=retry_config,
+            s3_staging_dir=s3_staging_dir,
             schema_name=schema_name,
             catalog_name=catalog_name,
             work_group=work_group,
+            poll_interval=poll_interval,
+            encryption_option=encryption_option,
+            kms_key=kms_key,
             kill_on_interrupt=kill_on_interrupt,
         )
+        self._max_workers = max_workers
         self._executor = ThreadPoolExecutor(max_workers=max_workers)
         self._arraysize = arraysize
         self._result_set_class = AthenaResultSet
@@ -76,39 +76,13 @@ class AsyncCursor(BaseCursor):
 
     def _description(
         self, query_id: str
-    ) -> Optional[
-        List[
-            Tuple[
-                Optional[Any],
-                Optional[Any],
-                None,
-                None,
-                Optional[Any],
-                Optional[Any],
-                Optional[Any],
-            ]
-        ]
-    ]:
+    ) -> Optional[List[Tuple[str, str, None, None, int, int, str]]]:
         result_set = self._collect_result_set(query_id)
         return result_set.description
 
     def description(
         self, query_id: str
-    ) -> "Future[\
-        Optional[\
-            List[\
-                Tuple[\
-                    Optional[Any],\
-                    Optional[Any],\
-                    None,\
-                    None,\
-                    Optional[Any],\
-                    Optional[Any],\
-                    Optional[Any],\
-                ]\
-            ]\
-        ]\
-    ]":
+    ) -> "Future[Optional[List[Tuple[str, str, None, None, int, int, str]]]]":
         return self._executor.submit(self._description, query_id)
 
     def query_execution(self, query_id: str) -> "Future[AthenaQueryExecution]":
@@ -135,7 +109,7 @@ class AsyncCursor(BaseCursor):
         s3_staging_dir: Optional[str] = None,
         cache_size: int = 0,
         cache_expiration_time: int = 0,
-    ) -> Tuple[str, "Future[Union[AthenaResultSet, AthenaPandasResultSet]]"]:
+    ) -> Tuple[str, "Future[Union[AthenaResultSet, Any]]"]:
         query_id = self._execute(
             operation,
             parameters=parameters,
@@ -148,7 +122,7 @@ class AsyncCursor(BaseCursor):
 
     def executemany(
         self, operation: str, seq_of_parameters: List[Optional[Dict[str, Any]]]
-    ):
+    ) -> None:
         raise NotSupportedError
 
     def cancel(self, query_id: str) -> "Future[None]":
