@@ -4,9 +4,9 @@ import sys
 import time
 from abc import ABCMeta, abstractmethod
 from datetime import datetime, timedelta, timezone
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Union
 
-from pyathena.converter import Converter
+from pyathena.converter import Converter, DefaultTypeConverter
 from pyathena.error import DatabaseError, OperationalError, ProgrammingError
 from pyathena.formatter import Formatter
 from pyathena.model import AthenaQueryExecution, AthenaTableMetadata
@@ -18,8 +18,10 @@ if TYPE_CHECKING:
 _logger = logging.getLogger(__name__)  # type: ignore
 
 
-class CursorIterator(object, metaclass=ABCMeta):
+class CursorIterator(metaclass=ABCMeta):
 
+    # https://docs.aws.amazon.com/athena/latest/APIReference/API_GetQueryResults.html
+    # Valid Range: Minimum value of 1. Maximum value of 1000.
     DEFAULT_FETCH_SIZE: int = 1000
 
     def __init__(self, **kwargs) -> None:
@@ -69,13 +71,11 @@ class CursorIterator(object, metaclass=ABCMeta):
         else:
             return row
 
-    next = __next__
-
     def __iter__(self):
         return self
 
 
-class BaseCursor(object, metaclass=ABCMeta):
+class BaseCursor(metaclass=ABCMeta):
 
     # https://docs.aws.amazon.com/athena/latest/APIReference/API_ListQueryExecutions.html
     # Valid Range: Minimum value of 0. Maximum value of 50.
@@ -87,6 +87,9 @@ class BaseCursor(object, metaclass=ABCMeta):
     def __init__(
         self,
         connection: "Connection",
+        converter: Converter,
+        formatter: Formatter,
+        retry_config: RetryConfig,
         s3_staging_dir: Optional[str],
         schema_name: Optional[str],
         catalog_name: Optional[str],
@@ -94,14 +97,14 @@ class BaseCursor(object, metaclass=ABCMeta):
         poll_interval: float,
         encryption_option: Optional[str],
         kms_key: Optional[str],
-        converter: Converter,
-        formatter: Formatter,
-        retry_config: RetryConfig,
         kill_on_interrupt: bool,
-        **kwargs
+        **kwargs,
     ) -> None:
         super(BaseCursor, self).__init__()
         self._connection = connection
+        self._converter = converter
+        self._formatter = formatter
+        self._retry_config = retry_config
         self._s3_staging_dir = s3_staging_dir
         self._schema_name = schema_name
         self._catalog_name = catalog_name
@@ -109,11 +112,12 @@ class BaseCursor(object, metaclass=ABCMeta):
         self._poll_interval = poll_interval
         self._encryption_option = encryption_option
         self._kms_key = kms_key
-        self._converter = converter
-        self._formatter = formatter
-        self._retry_config = retry_config
         self._kill_on_interrupt = kill_on_interrupt
 
+    @staticmethod
+    def get_default_converter(unload: bool = False) -> Union[DefaultTypeConverter, Any]:
+        return DefaultTypeConverter()
+
     @property
     def connection(self) -> "Connection":
         return self._connection
@@ -125,7 +129,7 @@ class BaseCursor(object, metaclass=ABCMeta):
                 self._connection.client.get_query_execution,
                 config=self._retry_config,
                 logger=_logger,
-                **request
+                **request,
             )
         except Exception as e:
             _logger.exception("Failed to get query execution.")
@@ -138,7 +142,8 @@ class BaseCursor(object, metaclass=ABCMeta):
         table_name: str,
         catalog_name: Optional[str] = None,
         schema_name: Optional[str] = None,
-    ):
+        logging_: bool = True,
+    ) -> AthenaTableMetadata:
         request = {
             "CatalogName": catalog_name if catalog_name else self._catalog_name,
             "DatabaseName": schema_name if schema_name else self._schema_name,
@@ -149,17 +154,30 @@ class BaseCursor(object, metaclass=ABCMeta):
                 self._connection.client.get_table_metadata,
                 config=self._retry_config,
                 logger=_logger,
-                **request
+                **request,
             )
         except Exception as e:
-            _logger.exception("Failed to get table metadata.")
+            if logging_:
+                _logger.exception("Failed to get table metadata.")
             raise OperationalError(*e.args) from e
         else:
             return AthenaTableMetadata(response)
 
-    def _batch_get_query_execution(
-        self, query_ids: List[str]
-    ) -> List[AthenaQueryExecution]:
+    def get_table_metadata(
+        self,
+        table_name: str,
+        catalog_name: Optional[str] = None,
+        schema_name: Optional[str] = None,
+        logging_: bool = True,
+    ) -> AthenaTableMetadata:
+        return self._get_table_metadata(
+            table_name=table_name,
+            catalog_name=catalog_name,
+            schema_name=schema_name,
+            logging_=logging_,
+        )
+
+    def _batch_get_query_execution(self, query_ids: List[str]) -> List[AthenaQueryExecution]:
         try:
             response = retry_api_call(
                 self.connection._client.batch_get_query_execution,
@@ -190,7 +208,7 @@ class BaseCursor(object, metaclass=ABCMeta):
                 self.connection._client.list_query_executions,
                 config=self._retry_config,
                 logger=_logger,
-                **request
+                **request,
             )
         except Exception as e:
             _logger.exception("Failed to list query executions.")
@@ -209,7 +227,7 @@ class BaseCursor(object, metaclass=ABCMeta):
         schema_name: Optional[str] = None,
         expression: Optional[str] = None,
         next_token: Optional[str] = None,
-    ):
+    ) -> Tuple[Optional[str], List[AthenaTableMetadata]]:
         request = self._build_list_table_metadata_request(
             max_results=max_results,
             catalog_name=catalog_name,
@@ -222,7 +240,7 @@ class BaseCursor(object, metaclass=ABCMeta):
                 self.connection._client.list_table_metadata,
                 config=self._retry_config,
                 logger=_logger,
-                **request
+                **request,
             )
         except Exception as e:
             _logger.exception("Failed to list table metadata.")
@@ -233,6 +251,28 @@ class BaseCursor(object, metaclass=ABCMeta):
                 for r in response.get("TableMetadataList", [])
             ]
 
+    def list_table_metadata(
+        self,
+        max_results: Optional[int] = None,
+        catalog_name: Optional[str] = None,
+        schema_name: Optional[str] = None,
+        expression: Optional[str] = None,
+    ) -> List[AthenaTableMetadata]:
+        metadata = []
+        next_token = None
+        while True:
+            next_token, response = self._list_table_metadata(
+                max_results=max_results,
+                catalog_name=catalog_name,
+                schema_name=schema_name,
+                expression=expression,
+                next_token=next_token,
+            )
+            metadata.extend(response)
+            if not next_token:
+                break
+        return metadata
+
     def __poll(self, query_id: str) -> AthenaQueryExecution:
         while True:
             query_execution = self._get_query_execution(query_id)
@@ -274,16 +314,10 @@ class BaseCursor(object, metaclass=ABCMeta):
             request["QueryExecutionContext"].update({"Catalog": self._catalog_name})
         if self._s3_staging_dir or s3_staging_dir:
             request["ResultConfiguration"].update(
-                {
-                    "OutputLocation": s3_staging_dir
-                    if s3_staging_dir
-                    else self._s3_staging_dir
-                }
+                {"OutputLocation": s3_staging_dir if s3_staging_dir else self._s3_staging_dir}
             )
         if self._work_group or work_group:
-            request.update(
-                {"WorkGroup": work_group if work_group else self._work_group}
-            )
+            request.update({"WorkGroup": work_group if work_group else self._work_group})
         if self._encryption_option:
             enc_conf = {
                 "EncryptionOption": self._encryption_option,
@@ -300,14 +334,10 @@ class BaseCursor(object, metaclass=ABCMeta):
         next_token: Optional[str] = None,
     ) -> Dict[str, Any]:
         request: Dict[str, Any] = {
-            "MaxResults": max_results
-            if max_results
-            else self.LIST_QUERY_EXECUTIONS_MAX_RESULTS
+            "MaxResults": max_results if max_results else self.LIST_QUERY_EXECUTIONS_MAX_RESULTS
         }
         if self._work_group or work_group:
-            request.update(
-                {"WorkGroup": work_group if work_group else self._work_group}
-            )
+            request.update({"WorkGroup": work_group if work_group else self._work_group})
         if next_token:
             request.update({"NextToken": next_token})
         return request
@@ -321,9 +351,7 @@ class BaseCursor(object, metaclass=ABCMeta):
         next_token: Optional[str] = None,
     ) -> Dict[str, Any]:
         request: Dict[str, Any] = {
-            "MaxResults": max_results
-            if max_results
-            else self.LIST_TABLE_METADATA_MAX_RESULTS,
+            "MaxResults": max_results if max_results else self.LIST_TABLE_METADATA_MAX_RESULTS,
             "CatalogName": catalog_name if catalog_name else self._catalog_name,
             "DatabaseName": schema_name if schema_name else self._schema_name,
         }
@@ -344,9 +372,7 @@ class BaseCursor(object, metaclass=ABCMeta):
         if cache_size == 0 and cache_expiration_time > 0:
             cache_size = sys.maxsize
         if cache_expiration_time > 0:
-            expiration_time = datetime.now(timezone.utc) - timedelta(
-                seconds=cache_expiration_time
-            )
+            expiration_time = datetime.now(timezone.utc) - timedelta(seconds=cache_expiration_time)
         else:
             expiration_time = datetime.now(timezone.utc)
         try:
@@ -382,9 +408,7 @@ class BaseCursor(object, metaclass=ABCMeta):
                 if query_id or next_token is None:
                     break
         except Exception:
-            _logger.warning(
-                "Failed to check the cache. Moving on without cache.", exc_info=True
-            )
+            _logger.warning("Failed to check the cache. Moving on without cache.", exc_info=True)
         return query_id
 
     def _execute(
@@ -399,9 +423,7 @@ class BaseCursor(object, metaclass=ABCMeta):
         query = self._formatter.format(operation, parameters)
         _logger.debug(query)
 
-        request = self._build_start_query_execution_request(
-            query, work_group, s3_staging_dir
-        )
+        request = self._build_start_query_execution_request(query, work_group, s3_staging_dir)
         query_id = self._find_previous_query_id(
             query,
             work_group,
@@ -414,7 +436,7 @@ class BaseCursor(object, metaclass=ABCMeta):
                     self._connection.client.start_query_execution,
                     config=self._retry_config,
                     logger=_logger,
-                    **request
+                    **request,
                 ).get("QueryExecutionId", None)
             except Exception as e:
                 _logger.exception("Failed to execute query.")
@@ -436,7 +458,7 @@ class BaseCursor(object, metaclass=ABCMeta):
     @abstractmethod
     def executemany(
         self, operation: str, seq_of_parameters: List[Optional[Dict[str, Any]]]
-    ):
+    ) -> None:
         raise NotImplementedError  # pragma: no cover
 
     @abstractmethod
@@ -450,7 +472,7 @@ class BaseCursor(object, metaclass=ABCMeta):
                 self._connection.client.stop_query_execution,
                 config=self._retry_config,
                 logger=_logger,
-                **request
+                **request,
             )
         except Exception as e:
             _logger.exception("Failed to cancel query.")
