@@ -7,11 +7,7 @@ from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type
 from boto3.session import Session
 
 from pyathena.common import BaseCursor
-from pyathena.converter import (
-    Converter,
-    DefaultPandasTypeConverter,
-    DefaultTypeConverter,
-)
+from pyathena.converter import Converter
 from pyathena.cursor import Cursor
 from pyathena.error import NotSupportedError
 from pyathena.formatter import DefaultParameterFormatter, Formatter
@@ -23,7 +19,7 @@ if TYPE_CHECKING:
 _logger = logging.getLogger(__name__)  # type: ignore
 
 
-class Connection(object):
+class Connection:
 
     _ENV_S3_STAGING_DIR: str = "AWS_ATHENA_S3_STAGING_DIR"
     _ENV_WORK_GROUP: str = "AWS_ATHENA_WORK_GROUP"
@@ -59,20 +55,23 @@ class Connection(object):
         profile_name: Optional[str] = None,
         role_arn: Optional[str] = None,
         role_session_name: str = "PyAthena-session-{0}".format(int(time.time())),
+        external_id: Optional[str] = None,
         serial_number: Optional[str] = None,
         duration_seconds: int = 3600,
         converter: Optional[Converter] = None,
         formatter: Optional[Formatter] = None,
         retry_config: Optional[RetryConfig] = None,
         cursor_class: Type[BaseCursor] = Cursor,
+        cursor_kwargs: Optional[Dict[str, Any]] = None,
         kill_on_interrupt: bool = True,
         session: Optional[Session] = None,
-        **kwargs
+        **kwargs,
     ) -> None:
         self._kwargs = {
             **kwargs,
             "role_arn": role_arn,
             "role_session_name": role_session_name,
+            "external_id": external_id,
             "serial_number": serial_number,
             "duration_seconds": duration_seconds,
         }
@@ -105,6 +104,7 @@ class Connection(object):
                     region_name=self.region_name,
                     role_arn=role_arn,
                     role_session_name=role_session_name,
+                    external_id=external_id,
                     serial_number=serial_number,
                     duration_seconds=duration_seconds,
                 )
@@ -132,7 +132,9 @@ class Connection(object):
                     }
                 )
             self._session = Session(
-                profile_name=self.profile_name, **self._session_kwargs
+                region_name=self.region_name,
+                profile_name=self.profile_name,
+                **self._session_kwargs,
             )
         self._client = self._session.client(
             "athena", region_name=self.region_name, **self._client_kwargs
@@ -141,24 +143,34 @@ class Connection(object):
         self._formatter = formatter if formatter else DefaultParameterFormatter()
         self._retry_config = retry_config if retry_config else RetryConfig()
         self.cursor_class = cursor_class
+        self.cursor_kwargs = cursor_kwargs if cursor_kwargs else dict()
         self.kill_on_interrupt = kill_on_interrupt
 
     def _assume_role(
         self,
         profile_name: Optional[str],
         region_name: Optional[str],
-        role_arn: Optional[str],
+        role_arn: str,
         role_session_name: str,
+        external_id: Optional[str],
         serial_number: Optional[str],
         duration_seconds: int,
     ) -> Dict[str, Any]:
-        session = Session(profile_name=profile_name, **self._session_kwargs)
+        session = Session(
+            region_name=region_name, profile_name=profile_name, **self._session_kwargs
+        )
         client = session.client("sts", region_name=region_name, **self._client_kwargs)
         request = {
             "RoleArn": role_arn,
             "RoleSessionName": role_session_name,
             "DurationSeconds": duration_seconds,
         }
+        if external_id:
+            request.update(
+                {
+                    "ExternalId": external_id,
+                }
+            )
         if serial_number:
             token_code = input("Enter the MFA code: ")
             request.update(
@@ -192,9 +204,7 @@ class Connection(object):
 
     @property
     def _session_kwargs(self) -> Dict[str, Any]:
-        return {
-            k: v for k, v in self._kwargs.items() if k in self._SESSION_PASSING_ARGS
-        }
+        return {k: v for k, v in self._kwargs.items() if k in self._SESSION_PASSING_ARGS}
 
     @property
     def _client_kwargs(self) -> Dict[str, Any]:
@@ -219,31 +229,26 @@ class Connection(object):
         self.close()
 
     def cursor(self, cursor: Optional[Type[BaseCursor]] = None, **kwargs) -> BaseCursor:
+        kwargs.update(self.cursor_kwargs)
         if not cursor:
             cursor = self.cursor_class
         converter = kwargs.pop("converter", self._converter)
         if not converter:
-            from pyathena.pandas.async_cursor import AsyncPandasCursor
-            from pyathena.pandas.cursor import PandasCursor
-
-            if cursor is PandasCursor or cursor is AsyncPandasCursor:
-                converter = DefaultPandasTypeConverter()
-            else:
-                converter = DefaultTypeConverter()
+            converter = cursor.get_default_converter(kwargs.get("unload", False))
         return cursor(
             connection=self,
-            s3_staging_dir=kwargs.pop("s3_staging_dir", self.s3_staging_dir),
-            poll_interval=kwargs.pop("poll_interval", self.poll_interval),
-            encryption_option=kwargs.pop("encryption_option", self.encryption_option),
-            kms_key=kwargs.pop("kms_key", self.kms_key),
             converter=converter,
             formatter=kwargs.pop("formatter", self._formatter),
             retry_config=kwargs.pop("retry_config", self._retry_config),
+            s3_staging_dir=kwargs.pop("s3_staging_dir", self.s3_staging_dir),
             schema_name=kwargs.pop("schema_name", self.schema_name),
             catalog_name=kwargs.pop("catalog_name", self.catalog_name),
             work_group=kwargs.pop("work_group", self.work_group),
+            poll_interval=kwargs.pop("poll_interval", self.poll_interval),
+            encryption_option=kwargs.pop("encryption_option", self.encryption_option),
+            kms_key=kwargs.pop("kms_key", self.kms_key),
             kill_on_interrupt=kwargs.pop("kill_on_interrupt", self.kill_on_interrupt),
-            **kwargs
+            **kwargs,
         )
 
     def close(self) -> None:
