@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 import logging
+import textwrap
+import uuid
 from abc import ABCMeta, abstractmethod
 from copy import deepcopy
 from datetime import date, datetime
@@ -7,12 +9,13 @@ from decimal import Decimal
 from typing import Any, Callable, Dict, Optional, Type, TypeVar
 
 from pyathena.error import ProgrammingError
+from pyathena.model import AthenaCompression, AthenaFileFormat
 
 _logger = logging.getLogger(__name__)  # type: ignore
 _T = TypeVar("_T", bound="Formatter")
 
 
-class Formatter(object, metaclass=ABCMeta):
+class Formatter(metaclass=ABCMeta):
     def __init__(
         self,
         mappings: Dict[Type[Any], Callable[[_T, Callable[[str], str], Any], Any]],
@@ -46,11 +49,39 @@ class Formatter(object, metaclass=ABCMeta):
         self.mappings.update(mappings)
 
     @abstractmethod
-    def format(
-        self, operation: str, parameters: Optional[Dict[str, Any]] = None
-    ) -> str:
+    def format(self, operation: str, parameters: Optional[Dict[str, Any]] = None) -> str:
         raise NotImplementedError  # pragma: no cover
 
+    @staticmethod
+    def wrap_unload(
+        operation: str,
+        s3_staging_dir: str,
+        format_: str = AthenaFileFormat.FILE_FORMAT_PARQUET,
+        compression: str = AthenaCompression.COMPRESSION_SNAPPY,
+    ):
+        if not operation or not operation.strip():
+            raise ProgrammingError("Query is none or empty.")
+
+        operation_upper = operation.strip().upper()
+        if operation_upper.startswith("SELECT") or operation_upper.startswith("WITH"):
+            now = datetime.utcnow().strftime("%Y%m%d")
+            location = f"{s3_staging_dir}unload/{now}/{str(uuid.uuid4())}/"
+            operation = textwrap.dedent(
+                f"""
+                UNLOAD (
+                \t{operation.strip()}
+                )
+                TO '{location}'
+                WITH (
+                \tformat = '{format_}',
+                \tcompression = '{compression}'
+                )
+                """
+            )
+        else:
+            location = None
+        return operation, location
+
 
 def _escape_presto(val: str) -> str:
     """ParamEscaper
@@ -76,9 +107,7 @@ def _format_none(formatter: Formatter, escaper: Callable[[str], str], val: Any)
     return "null"
 
 
-def _format_default(
-    formatter: Formatter, escaper: Callable[[str], str], val: Any
-) -> Any:
+def _format_default(formatter: Formatter, escaper: Callable[[str], str], val: Any) -> Any:
     return val
 
 
@@ -86,9 +115,7 @@ def _format_date(formatter: Formatter, escaper: Callable[[str], str], val: Any)
     return "DATE '{0}'".format(val.strftime("%Y-%m-%d"))
 
 
-def _format_datetime(
-    formatter: Formatter, escaper: Callable[[str], str], val: Any
-) -> Any:
+def _format_datetime(formatter: Formatter, escaper: Callable[[str], str], val: Any) -> Any:
     return "TIMESTAMP '{0}'".format(val.strftime("%Y-%m-%d %H:%M:%S.%f")[:-3])
 
 
@@ -126,15 +153,11 @@ def _format_seq(formatter: Formatter, escaper: Callable[[str], str], val: Any) -
     return "({0})".format(", ".join(results))
 
 
-def _format_decimal(
-    formatter: Formatter, escaper: Callable[[str], str], val: Any
-) -> Any:
+def _format_decimal(formatter: Formatter, escaper: Callable[[str], str], val: Any) -> Any:
     return "DECIMAL {0}".format(escaper("{0:f}".format(val)))
 
 
-_DEFAULT_FORMATTERS: Dict[
-    Type[Any], Callable[[Formatter, Callable[[str], str], Any], Any]
-] = {
+_DEFAULT_FORMATTERS: Dict[Type[Any], Callable[[Formatter, Callable[[str], str], Any], Any]] = {
     type(None): _format_none,
     date: _format_date,
     datetime: _format_datetime,
@@ -155,15 +178,16 @@ class DefaultParameterFormatter(Formatter):
             mappings=deepcopy(_DEFAULT_FORMATTERS), default=None
         )
 
-    def format(
-        self, operation: str, parameters: Optional[Dict[str, Any]] = None
-    ) -> str:
+    def format(self, operation: str, parameters: Optional[Dict[str, Any]] = None) -> str:
         if not operation or not operation.strip():
             raise ProgrammingError("Query is none or empty.")
         operation = operation.strip()
 
-        if operation.upper().startswith("SELECT") or operation.upper().startswith(
-            "WITH"
+        operation_upper = operation.upper()
+        if (
+            operation_upper.startswith("SELECT")
+            or operation_upper.startswith("WITH")
+            or operation_upper.startswith("INSERT")
         ):
             escaper = _escape_presto
         else:
@@ -180,8 +204,7 @@ class DefaultParameterFormatter(Formatter):
                     kwargs.update({k: func(self, escaper, v)})
             else:
                 raise ProgrammingError(
-                    "Unsupported parameter "
-                    + "(Support for dict only): {0}".format(parameters)
+                    "Unsupported parameter " + "(Support for dict only): {0}".format(parameters)
                 )
 
         return (operation % kwargs).strip() if kwargs is not None else operation.strip()
