@@ -1,14 +1,15 @@
 # -*- coding: utf-8 -*-
 import logging
+import re
 from datetime import datetime
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List, Optional, Pattern
 
 from pyathena.error import DataError
 
 _logger = logging.getLogger(__name__)  # type: ignore
 
 
-class AthenaQueryExecution(object):
+class AthenaQueryExecution:
 
     STATE_QUEUED: str = "QUEUED"
     STATE_RUNNING: str = "RUNNING"
@@ -42,17 +43,11 @@ class AthenaQueryExecution(object):
             raise DataError("KeyError `Status`")
         self._state: Optional[str] = status.get("State", None)
         self._state_change_reason: Optional[str] = status.get("StateChangeReason", None)
-        self._completion_date_time: Optional[datetime] = status.get(
-            "CompletionDateTime", None
-        )
-        self._submission_date_time: Optional[datetime] = status.get(
-            "SubmissionDateTime", None
-        )
+        self._completion_date_time: Optional[datetime] = status.get("CompletionDateTime", None)
+        self._submission_date_time: Optional[datetime] = status.get("SubmissionDateTime", None)
 
         statistics = query_execution.get("Statistics", {})
-        self._data_scanned_in_bytes: Optional[int] = statistics.get(
-            "DataScannedInBytes", None
-        )
+        self._data_scanned_in_bytes: Optional[int] = statistics.get("DataScannedInBytes", None)
         self._engine_execution_time_in_millis: Optional[int] = statistics.get(
             "EngineExecutionTimeInMillis", None
         )
@@ -68,17 +63,13 @@ class AthenaQueryExecution(object):
         self._service_processing_time_in_millis: Optional[int] = statistics.get(
             "ServiceProcessingTimeInMillis", None
         )
-        self._data_manifest_location: Optional[str] = statistics.get(
-            "DataManifestLocation", None
-        )
+        self._data_manifest_location: Optional[str] = statistics.get("DataManifestLocation", None)
 
         result_conf = query_execution.get("ResultConfiguration", {})
         self._output_location: Optional[str] = result_conf.get("OutputLocation", None)
 
         encryption_conf = result_conf.get("EncryptionConfiguration", {})
-        self._encryption_option: Optional[str] = encryption_conf.get(
-            "EncryptionOption", None
-        )
+        self._encryption_option: Optional[str] = encryption_conf.get("EncryptionOption", None)
         self._kms_key: Optional[str] = encryption_conf.get("KmsKey", None)
 
         self._work_group: Optional[str] = query_execution.get("WorkGroup", None)
@@ -160,7 +151,7 @@ class AthenaQueryExecution(object):
         return self._work_group
 
 
-class AthenaTableMetadataColumn(object):
+class AthenaTableMetadataColumn:
     def __init__(self, response):
         self._name: Optional[str] = response.get("Name", None)
         self._type: Optional[str] = response.get("Type", None)
@@ -179,7 +170,7 @@ class AthenaTableMetadataColumn(object):
         return self._comment
 
 
-class AthenaTableMetadataPartitionKey(object):
+class AthenaTableMetadataPartitionKey:
     def __init__(self, response):
         self._name: Optional[str] = response.get("Name", None)
         self._type: Optional[str] = response.get("Type", None)
@@ -198,7 +189,7 @@ class AthenaTableMetadataPartitionKey(object):
         return self._comment
 
 
-class AthenaTableMetadata(object):
+class AthenaTableMetadata:
     def __init__(self, response):
         table_metadata = response.get("TableMetadata", None)
         if not table_metadata:
@@ -206,9 +197,7 @@ class AthenaTableMetadata(object):
 
         self._name: Optional[str] = table_metadata.get("Name", None)
         self._create_time: Optional[datetime] = table_metadata.get("CreateTime", None)
-        self._last_access_time: Optional[datetime] = table_metadata.get(
-            "LastAccessTime", None
-        )
+        self._last_access_time: Optional[datetime] = table_metadata.get("LastAccessTime", None)
         self._table_type: Optional[str] = table_metadata.get("TableType", None)
 
         columns = table_metadata.get("Columns", [])
@@ -267,42 +256,124 @@ class AthenaTableMetadata(object):
     def output_format(self) -> Optional[str]:
         return self._parameters.get("outputformat", None)
 
+    @property
+    def row_format(self) -> Optional[str]:
+        serde = self.serde_serialization_lib
+        if serde:
+            return f"SERDE '{serde}'"
+        return None
+
+    @property
+    def file_format(self) -> Optional[str]:
+        input = self.input_format
+        output = self.output_format
+        if input and output:
+            return f"INPUTFORMAT '{input}' OUTPUTFORMAT '{output}'"
+        return None
+
     @property
     def serde_serialization_lib(self) -> Optional[str]:
-        return self._parameters.get("'serde.serialization.lib", None)
+        return self._parameters.get("serde.serialization.lib", None)
+
+    @property
+    def compression(self) -> Optional[str]:
+        if "write.compression" in self._parameters:  # text or json
+            return self._parameters["write.compression"]
+        elif "serde.param.write.compression" in self._parameters:  # text or json
+            return self._parameters["serde.param.write.compression"]
+        elif "parquet.compress" in self._parameters:  # parquet
+            return self._parameters["parquet.compress"]
+        elif "orc.compress" in self._parameters:  # orc
+            return self._parameters["orc.compress"]
+        else:
+            return None
+
+    @property
+    def serde_properties(self) -> Dict[str, str]:
+        return {
+            k.replace("serde.param.", ""): v
+            for k, v in self._parameters.items()
+            if k.startswith("serde.param.")
+        }
+
+    @property
+    def table_properties(self) -> Dict[str, str]:
+        return {k: v for k, v in self._parameters.items() if not k.startswith("serde.param.")}
 
 
-class AthenaRowFormat(object):
+class AthenaFileFormat:
 
-    ROW_FORMAT_PARQUET: str = "parquet"
-    ROW_FORMAT_ORC: str = "orc"
-    ROW_FORMAT_CSV: str = "csv"
-    ROW_FORMAT_JSON: str = "json"
-    ROW_FORMAT_AVRO: str = "avro"
+    FILE_FORMAT_SEQUENCEFILE: str = "SEQUENCEFILE"
+    FILE_FORMAT_TEXTFILE: str = "TEXTFILE"
+    FILE_FORMAT_RCFILE: str = "RCFILE"
+    FILE_FORMAT_ORC: str = "ORC"
+    FILE_FORMAT_PARQUET: str = "PARQUET"
+    FILE_FORMAT_AVRO: str = "AVRO"
+    FILE_FORMAT_ION: str = "ION"
 
     @staticmethod
-    def is_valid(value: Optional[str]) -> bool:
-        return value in [
-            AthenaRowFormat.ROW_FORMAT_PARQUET,
-            AthenaRowFormat.ROW_FORMAT_ORC,
-            AthenaRowFormat.ROW_FORMAT_CSV,
-            AthenaRowFormat.ROW_FORMAT_JSON,
-            AthenaRowFormat.ROW_FORMAT_AVRO,
-        ]
-
-
-class AthenaCompression(object):
-
-    COMPRESSION_SNAPPY: str = "snappy"
-    COMPRESSION_ZLIB: str = "zlib"
-    COMPRESSION_LZO: str = "lzo"
-    COMPRESSION_GZIP: str = "gzip"
+    def is_parquet(value: str) -> bool:
+        return value.upper() == AthenaFileFormat.FILE_FORMAT_PARQUET
 
     @staticmethod
-    def is_valid(value: Optional[str]) -> bool:
-        return value in [
+    def is_orc(value: str) -> bool:
+        return value.upper() == AthenaFileFormat.FILE_FORMAT_ORC
+
+
+class AthenaRowFormatSerde:
+
+    PATTERN_ROW_FORMAT_SERDE: Pattern[str] = re.compile(r"^(?i:serde) '(?P<serde>.+)'$")
+
+    ROW_FORMAT_SERDE_CSV: str = "org.apache.hadoop.hive.serde2.OpenCSVSerde"
+    ROW_FORMAT_SERDE_REGEX: str = "org.apache.hadoop.hive.serde2.RegexSerDe"
+    ROW_FORMAT_SERDE_LAZY_SIMPLE: str = "org.apache.hadoop.hive.serde2.lazy.LazySimpleSerDe"
+    ROW_FORMAT_SERDE_CLOUD_TRAIL: str = "com.amazon.emr.hive.serde.CloudTrailSerde"
+    ROW_FORMAT_SERDE_GROK: str = "com.amazonaws.glue.serde.GrokSerDe"
+    ROW_FORMAT_SERDE_JSON: str = "org.openx.data.jsonserde.JsonSerDe"
+    ROW_FORMAT_SERDE_JSON_HCATALOG: str = "org.apache.hive.hcatalog.data.JsonSerDe"
+    ROW_FORMAT_SERDE_PARQUET: str = "org.apache.hadoop.hive.ql.io.parquet.serde.ParquetHiveSerDe"
+    ROW_FORMAT_SERDE_ORC: str = "org.apache.hadoop.hive.ql.io.orc.OrcSerde"
+    ROW_FORMAT_SERDE_AVRO: str = "org.apache.hadoop.hive.serde2.avro.AvroSerDe"
+
+    @staticmethod
+    def is_parquet(value: str) -> bool:
+        match = AthenaRowFormatSerde.PATTERN_ROW_FORMAT_SERDE.search(value)
+        if match:
+            serde = match.group("serde")
+            if serde == AthenaRowFormatSerde.ROW_FORMAT_SERDE_PARQUET:
+                return True
+        return False
+
+    @staticmethod
+    def is_orc(value: str) -> bool:
+        match = AthenaRowFormatSerde.PATTERN_ROW_FORMAT_SERDE.search(value)
+        if match:
+            serde = match.group("serde")
+            if serde == AthenaRowFormatSerde.ROW_FORMAT_SERDE_ORC:
+                return True
+        return False
+
+
+class AthenaCompression:
+
+    COMPRESSION_BZIP2: str = "BZIP2"
+    COMPRESSION_DEFLATE: str = "DEFLATE"
+    COMPRESSION_GZIP: str = "GZIP"
+    COMPRESSION_LZ4: str = "LZ4"
+    COMPRESSION_LZO: str = "LZO"
+    COMPRESSION_SNAPPY: str = "SNAPPY"
+    COMPRESSION_ZLIB: str = "ZLIB"
+    COMPRESSION_ZSTD: str = "ZSTD"
+
+    @staticmethod
+    def is_valid(value: str) -> bool:
+        return value.upper() in [
+            AthenaCompression.COMPRESSION_BZIP2,
+            AthenaCompression.COMPRESSION_DEFLATE,
+            AthenaCompression.COMPRESSION_GZIP,
+            AthenaCompression.COMPRESSION_LZ4,
+            AthenaCompression.COMPRESSION_LZO,
             AthenaCompression.COMPRESSION_SNAPPY,
             AthenaCompression.COMPRESSION_ZLIB,
-            AthenaCompression.COMPRESSION_LZO,
-            AthenaCompression.COMPRESSION_GZIP,
+            AthenaCompression.COMPRESSION_ZSTD,
         ]
