@@ -20,7 +20,7 @@ from pyathena.common import CursorIterator
 from pyathena.converter import Converter
 from pyathena.error import DataError, OperationalError, ProgrammingError
 from pyathena.model import AthenaQueryExecution
-from pyathena.util import RetryConfig, retry_api_call
+from pyathena.util import RetryConfig, parse_output_location, retry_api_call
 
 if TYPE_CHECKING:
     from pyathena.connection import Connection
@@ -43,8 +43,11 @@ class AthenaResultSet(CursorIterator):
         self._query_execution: Optional[AthenaQueryExecution] = query_execution
         assert self._query_execution, "Required argument `query_execution` not found."
         self._retry_config = retry_config
+        self._client = connection.session.client(
+            "s3", region_name=connection.region_name, **connection._client_kwargs
+        )
 
-        self._meta_data: Optional[Tuple[Any, ...]] = None
+        self._metadata: Optional[Tuple[Dict[str, Any], ...]] = None
         self._rows: Deque[
             Union[Tuple[Optional[Any], ...], Dict[Any, Optional[Any]]]
         ] = collections.deque()
@@ -171,35 +174,29 @@ class AthenaResultSet(CursorIterator):
     @property
     def description(
         self,
-    ) -> Optional[
-        List[
-            Tuple[
-                Optional[Any],
-                Optional[Any],
-                None,
-                None,
-                Optional[Any],
-                Optional[Any],
-                Optional[Any],
-            ]
-        ]
-    ]:
-        if self._meta_data is None:
+    ) -> Optional[List[Tuple[str, str, None, None, int, int, str]]]:
+        if self._metadata is None:
             return None
         return [
             (
-                m.get("Name", None),
-                m.get("Type", None),
+                m["Name"],
+                m["Type"],
                 None,
                 None,
-                m.get("Precision", None),
-                m.get("Scale", None),
-                m.get("Nullable", None),
+                m["Precision"],
+                m["Scale"],
+                m["Nullable"],
             )
-            for m in self._meta_data
+            for m in self._metadata
         ]
 
-    def __fetch(self, next_token: Optional[str] = None):
+    @property
+    def connection(self) -> "Connection":
+        if self.is_closed:
+            raise ProgrammingError("AthenaResultSet is closed.")
+        return cast("Connection", self._connection)
+
+    def __fetch(self, next_token: Optional[str] = None) -> Dict[str, Any]:
         if not self.query_id:
             raise ProgrammingError("QueryExecutionId is none or empty.")
         if self.state != AthenaQueryExecution.STATE_SUCCEEDED:
@@ -213,31 +210,32 @@ class AthenaResultSet(CursorIterator):
         if next_token:
             request.update({"NextToken": next_token})
         try:
-            connection = cast("Connection", self._connection)
             response = retry_api_call(
-                connection.client.get_query_results,
+                self.connection.client.get_query_results,
                 config=self._retry_config,
                 logger=_logger,
-                **request
+                **request,
             )
         except Exception as e:
             _logger.exception("Failed to fetch result set.")
             raise OperationalError(*e.args) from e
         else:
-            return response
+            return cast(Dict[str, Any], response)
 
-    def _fetch(self):
+    def _fetch(self) -> None:
         if not self._next_token:
             raise ProgrammingError("NextToken is none or empty.")
         response = self.__fetch(self._next_token)
         self._process_rows(response)
 
-    def _pre_fetch(self):
+    def _pre_fetch(self) -> None:
         response = self.__fetch()
-        self._process_meta_data(response)
+        self._process_metadata(response)
         self._process_rows(response)
 
-    def fetchone(self):
+    def fetchone(
+        self,
+    ) -> Optional[Union[Tuple[Optional[Any], ...], Dict[Any, Optional[Any]]]]:
         if not self._rows and self._next_token:
             self._fetch()
         if not self._rows:
@@ -248,7 +246,9 @@ class AthenaResultSet(CursorIterator):
             self._rownumber += 1
             return self._rows.popleft()
 
-    def fetchmany(self, size: Optional[int] = None):
+    def fetchmany(
+        self, size: Optional[int] = None
+    ) -> List[Union[Tuple[Optional[Any], ...], Dict[Any, Optional[Any]]]]:
         if not size or size <= 0:
             size = self._arraysize
         rows = []
@@ -260,7 +260,9 @@ class AthenaResultSet(CursorIterator):
                 break
         return rows
 
-    def fetchall(self):
+    def fetchall(
+        self,
+    ) -> List[Union[Tuple[Optional[Any], ...], Dict[Any, Optional[Any]]]]:
         rows = []
         while True:
             row = self.fetchone()
@@ -270,28 +272,26 @@ class AthenaResultSet(CursorIterator):
                 break
         return rows
 
-    def _process_meta_data(self, response: Dict[str, Any]) -> None:
+    def _process_metadata(self, response: Dict[str, Any]) -> None:
         result_set = response.get("ResultSet", None)
         if not result_set:
             raise DataError("KeyError `ResultSet`")
-        meta_data = result_set.get("ResultSetMetadata", None)
-        if not meta_data:
+        metadata = result_set.get("ResultSetMetadata", None)
+        if not metadata:
             raise DataError("KeyError `ResultSetMetadata`")
-        column_info = meta_data.get("ColumnInfo", None)
+        column_info = metadata.get("ColumnInfo", None)
         if column_info is None:
             raise DataError("KeyError `ColumnInfo`")
-        self._meta_data = tuple(column_info)
+        self._metadata = tuple(column_info)
 
     def _get_rows(
-        self, offset: int, meta_data: Tuple[Any, ...], rows: List[Dict[str, Any]]
+        self, offset: int, metadata: Tuple[Any, ...], rows: List[Dict[str, Any]]
     ) -> List[Union[Tuple[Optional[Any], ...], Dict[Any, Optional[Any]]]]:
         return [
             tuple(
                 [
-                    self._converter.convert(
-                        meta.get("Type", None), row.get("VarCharValue", None)
-                    )
-                    for meta, row in zip(meta_data, rows[i].get("Data", []))
+                    self._converter.convert(meta.get("Type", None), row.get("VarCharValue", None))
+                    for meta, row in zip(metadata, rows[i].get("Data", []))
                 ]
             )
             for i in range(offset, len(rows))
@@ -306,24 +306,57 @@ class AthenaResultSet(CursorIterator):
             raise DataError("KeyError `Rows`")
         processed_rows = []
         if len(rows) > 0:
-            offset = (
-                1
-                if not self._next_token and self._is_first_row_column_labels(rows)
-                else 0
-            )
-            meta_data = cast(Tuple[Any, ...], self._meta_data)
-            processed_rows = self._get_rows(offset, meta_data, rows)
+            offset = 1 if not self._next_token and self._is_first_row_column_labels(rows) else 0
+            metadata = cast(Tuple[Any, ...], self._metadata)
+            processed_rows = self._get_rows(offset, metadata, rows)
         self._rows.extend(processed_rows)
         self._next_token = response.get("NextToken", None)
 
     def _is_first_row_column_labels(self, rows: List[Dict[str, Any]]) -> bool:
         first_row_data = rows[0].get("Data", [])
-        meta_data = cast(Tuple[Any, Any], self._meta_data)
-        for meta, data in zip(meta_data, first_row_data):
+        metadata = cast(Tuple[Any, Any], self._metadata)
+        for meta, data in zip(metadata, first_row_data):
             if meta.get("Name", None) != data.get("VarCharValue", None):
                 return False
         return True
 
+    def _get_content_length(self) -> int:
+        if not self.output_location:
+            raise ProgrammingError("OutputLocation is none or empty.")
+        bucket, key = parse_output_location(self.output_location)
+        try:
+            response = retry_api_call(
+                self._client.head_object,
+                config=self._retry_config,
+                logger=_logger,
+                Bucket=bucket,
+                Key=key,
+            )
+        except Exception as e:
+            _logger.exception("Failed to get content length.")
+            raise OperationalError(*e.args) from e
+        else:
+            return cast(int, response["ContentLength"])
+
+    def _read_data_manifest(self) -> List[str]:
+        if not self.data_manifest_location:
+            raise ProgrammingError("DataManifestLocation is none or empty.")
+        bucket, key = parse_output_location(self.data_manifest_location)
+        try:
+            response = retry_api_call(
+                self._client.get_object,
+                config=self._retry_config,
+                logger=_logger,
+                Bucket=bucket,
+                Key=key,
+            )
+        except Exception as e:
+            _logger.exception(f"Failed to read {bucket}/{key}.")
+            raise OperationalError(*e.args) from e
+        else:
+            manifest: str = response["Body"].read().decode("utf-8").strip()
+            return manifest.split("\n") if manifest else []
+
     @property
     def is_closed(self) -> bool:
         return self._connection is None
@@ -331,7 +364,7 @@ class AthenaResultSet(CursorIterator):
     def close(self) -> None:
         self._connection = None
         self._query_execution = None
-        self._meta_data = None
+        self._metadata = None
         self._rows.clear()
         self._next_token = None
         self._rownumber = None
@@ -349,7 +382,7 @@ class AthenaDictResultSet(AthenaResultSet):
     dict_type: Type[Any] = dict
 
     def _get_rows(
-        self, offset: int, meta_data: Tuple[Any, ...], rows: List[Dict[str, Any]]
+        self, offset: int, metadata: Tuple[Any, ...], rows: List[Dict[str, Any]]
     ) -> List[Union[Tuple[Optional[Any], ...], Dict[Any, Optional[Any]]]]:
         return [
             self.dict_type(
@@ -360,14 +393,14 @@ class AthenaDictResultSet(AthenaResultSet):
                             meta.get("Type", None), row.get("VarCharValue", None)
                         ),
                     )
-                    for meta, row in zip(meta_data, rows[i].get("Data", []))
+                    for meta, row in zip(metadata, rows[i].get("Data", []))
                 ]
             )
             for i in range(offset, len(rows))
         ]
 
 
-class WithResultSet(object):
+class WithResultSet:
     def __init__(self):
         super(WithResultSet, self).__init__()
 
@@ -394,19 +427,7 @@ class WithResultSet(object):
     @property
     def description(
         self,
-    ) -> Optional[
-        List[
-            Tuple[
-                Optional[Any],
-                Optional[Any],
-                None,
-                None,
-                Optional[Any],
-                Optional[Any],
-                Optional[Any],
-            ]
-        ]
-    ]:
+    ) -> Optional[List[Tuple[str, str, None, None, int, int, str]]]:
         if not self.has_result_set:
             return None
         result_set = cast(AthenaResultSet, self.result_set)
