@@ -39,7 +39,7 @@ def synchronized(wrapped: Callable[..., Any]) -> Any:
     return _wrapper
 
 
-class RetryConfig(object):
+class RetryConfig:
     def __init__(
         self,
         exceptions: Iterable[str] = (
@@ -63,7 +63,7 @@ def retry_api_call(
     config: RetryConfig,
     logger: logging.Logger = None,
     *args,
-    **kwargs
+    **kwargs,
 ) -> Any:
     retry = tenacity.Retrying(
         retry=retry_if_exception(
